<?php
/*
 * Plugin Name: Bicycle Theory Locator
 * Version: 1.0.9
 * Description: Map-based locator component for Bicycle Theory projects.
 * Author: Bicycle Theory
 * Author URI: https://bicycletheory.com/
 * Requires at least: 4.9
 * Tested up to: 6.1.1
 *
 * Text Domain: bt
 * Domain Path: /lang/
 * License: This plugin contains copyrighted software and is only to be used for Bicycle Theory projects.
 *
 * @package WordPress
 * @author Jeremy Burgeson
 * @since 1.0.0
 */

/**
 * Prevent direct access to plugin file.
 */
if ( ! defined( 'ABSPATH' ) ) exit;



/**
 * Constants
 */
define( 'BT_LOCATOR_VERSION', '1.0.9' );


/**
 * Include component files.
 */
require_once __DIR__ . '/lib/helpers.php';
require_once __DIR__ . '/lib/wp-query-bt-location.php';


/**
 * Check to see if the current site (theme via theme-supports) uses locator, and optionally get a config item set in
 * the theme setup.
 *
 * @param string $config_item
 *
 * @return mixed false if no support, true if support but no requested config item, else the requested config item
 */
function bt_use_locator( $config_item = '' ) {
	$locator_config = get_theme_support( 'bt-locator' );

	if ( empty( $locator_config ) ) {
		return false;
	}

	// get a config item if requested
	if ( ! empty( $config_item ) ) {
		foreach ( $locator_config as $config ) {
			//error_log( __METHOD__ . ' -- config = ' . print_r( $config, 1 ) );
			if ( array_key_exists( $config_item, $config ) ) {
				// found the config item in question
				//error_log( __METHOD__ . ' -- found = ' . $config[$config_item] );
				return $config[$config_item];
			}
		}
	}

	// theme support exists but no config requested
	return true;
}


/**
 * Determine and return plugin URL (front end) path, which is weird because of mu-plugin or local server
 *
 * @return string
 */
function bt_locator_plugin_url() {
	return trailingslashit( str_replace( '/plugins', '/mu-plugins', plugins_url() ) )
	       . trailingslashit( basename( __DIR__ ) );
}


/**
 * Load JS for locator and google maps
 */
add_action('wp_enqueue_scripts', function () {
	$plugin_url = bt_locator_plugin_url();

	// things to do on the front end
	if ( ! is_admin() ) {

		$gmaps_api_key = bt_use_locator( 'gmaps_api_key' );

		// google maps
		if ( $gmaps_api_key && bt_load_gmaps() ) {

			// include google maps api script
			wp_enqueue_script(
				'google-maps',
				'https://maps.googleapis.com/maps/api/js?key=' . $gmaps_api_key,
				[],
				null,
				true
			);

			// include our locator js
			wp_enqueue_script(
				'bt_locator',
				$plugin_url . 'bt-locator.js',
				[ 'jquery' ],
				BT_LOCATOR_VERSION,
				true
			);

			// set up ajax data for locator
			wp_localize_script( 'bt_locator', 'locator_data', [
				'ajax_url' => admin_url( 'admin-ajax.php' ),
				'nonce' => wp_create_nonce( 'locator_nonce' ),
				'first' => 1,
			] );

			// global google map vars
			wp_localize_script( 'bt_locator', 'gmap', [
				'map' => null,
				'options' => null,
				'info_window' => null,
				'point_bounds' => null,
				'marker' => null,
				'markers' => null,
				'initial_lat' => bt_use_locator( 'map_initial_lat' ),
				'initial_lng' => bt_use_locator( 'map_initial_lng' ),
				'initial_zoom' => bt_use_locator( 'map_initial_zoom' ),
			] );

		}

	}
}, 9977);


/**
 * Determine if we should include google maps on the current view. Checks theme-supports data for any pages and post
 * types which should.
 *
 * @return bool
 */
function bt_load_gmaps() {
	$load_map_on = bt_use_locator( 'load_map_on' );

	if ( empty( $load_map_on ) ) {
		return false;
	}

	switch ( get_post_type() ) {
		case 'page':
			return in_array( get_post_field( 'post_name' ), $load_map_on['pages'] );
		default:
			return in_array( get_post_type(), $load_map_on['post_types'] );
	}
}


/**
 * Hook into ACF Save Post (after post data is saved) so ast to:
 * - set simple lat/lng meta from street address
 *
 * @param int $post_id
 */
// hook into ACF save post action for post types used by locator
add_action( 'acf/save_post', function( $post_id ) {
	// run for location post types
	if ( in_array( get_post_type( $post_id ), bt_locator_post_types() ) ) {
		// get ACF-based street address for its lat/lng elements
		$address = bt_address_object_for_post( $post_id );

		// set lat/lng in simple post meta
		if ( isset( $address->lat ) && $address->lat && isset( $address->lng ) && $address->lng) {
			update_post_meta( $post_id, 'lat', $address->lat );
			update_post_meta( $post_id, 'lng', $address->lng );
		}
	}
}, 9977 );


/**
 * Generate and return an array of Retail Attributes (its own post type)
 *
 * @return array All WP_Post objects returned from the WP_Query.
 */
/*
function locator_attributes() {
	// get and return location attributes
	$wp_query = new \WP_Query([
		'post_type' => 'location-attribute',
		'nopaging' => true,
		'orderby' => 'title',
		'order' => 'DESC',
		'post_status' => 'publish',
	]);
	return $wp_query->posts;
}
*/


/**
 * CONVERTING TO A DATA NORMALIZATION ROUTINE SINCE WE'RE USING SIMPLE ATTRIBUTES ONLY.
 * BUT WE MAY WANT TO STORE SEARCHES AND RESULTS IN CACHE LIKE VY.
 *
 * Fetch various options for locations, such as all available cities and years, via cached/databased options.
 * Will regenerate the data if it's stale or doesn't exist. This function also normalizes some data, like making sure
 * there are simple lat/lng meta bits to facilitate zip/radius querying.
 *
 * @param bool $force_recalculate If true, the recalculation of data is forced to happen.
 *
 * @return array Keys are option slugs like 'attributes' with simple array values or key/value pairs.
 */
function bt_locator_data_normalization( $force_recalculate = true ) {
    // get the location post type and make sure we have one
	if ( empty( $location_post_type = bt_locator_post_types() ) ) {
		error_log( __METHOD__ . ': no location post type set in theme setup.' );
		return [];
	}

	// check last stored value and timestamp
	$now = time();
	$threshold = 86400; // 24 hours
	$last_updated = get_option( 'bt_locator_cache_last_updated', 0 );
	$past_threshold = ( $now - $last_updated ) > $threshold;

	// get the various cached info
    $cache_items = [ /* 'attributes' */];
    $missing_something = false;
    foreach ( $cache_items as $cache_item ) {
        $$cache_item = get_option( 'bt_locator_' . $cache_item, [] );
	    $missing_something = $missing_something || empty( $$cache_item );
    }

	$attributes = [];

	// recalculate when needed
	if ( $force_recalculate || $past_threshold || $missing_something ) {
		ini_set('memory_limit', '512M');
		ini_set('max_execution_time', '120');

        // process attributes, including deletion of existing meta/normalized data in advance of insertion of current data
		/*
	    global $wpdb;
		$wpdb->query( "DELETE FROM $wpdb->postmeta WHERE (meta_key LIKE 'location-attribute-%' OR meta_key LIKE '_location-attribute-%')" );
        foreach ( locator_attributes() as $attribute ) {
            error_log(__FUNCTION__.' -- Processing attribute '.$attribute->post_title);
            $attributes[$attribute->post_name] = $attribute->post_title;
        }
		*/

        // query for all location location records
		$args = [
			'post_type' => $location_post_type,
			'nopaging' => true,
			'post_status' => 'publish',
		];
		$wp_query = new \WP_Query($args);

		// loop records to collect the data we need
		foreach ( $wp_query->posts as $post ) {

		    // handle address
			$address = bt_address_object_for_post( $post->ID );

            // handle lat/lng
            if ( isset( $address->lat ) && $address->lat && isset( $address->lng ) && $address->lng) {
			    $meta_lat = get_post_meta( $post->ID ,'lat', true );
			    $meta_lng = get_post_meta( $post->ID, 'lng', true );
			    if ( BT_ENV != 'prod' && ( ! $meta_lat || ! $meta_lng ) ) {
			        error_log(__FUNCTION__.' -- Missing lat/lng for post ' . $post->ID . ' / ' . $post->post_name);
                }
			    if ( number_format( $address->lat, 6) != number_format( $meta_lat, 6 ) ) {
				    error_log(__FUNCTION__.' -- Updating lat for post ' . $post->ID . ' / ' . $post->post_name);
				    update_post_meta( $post->ID, 'lat', $address->lat );
                }
	            if ( number_format( $address->lng, 6) != number_format( $meta_lng, 6 ) ) {
		            error_log(__FUNCTION__.' -- Updating lng for post ' . $post->ID . ' / ' . $post->post_name);
		            update_post_meta( $post->ID, 'lng', $address->lng );
                }
            }

            // save address back to the post meta, forcing as an array, so that we don't end up with objects in database.
            // objects were ok until we started using SearchWP, and its Term Highlight extension, which dies when it tries a string operation on objects.
			if ( is_object( get_field( 'street_address', $post->ID ) ) ) {
			    error_log(__FUNCTION__.' -- Address for post '.$post->ID.' was an object! Saving to array...');
				update_field( 'street_address', (array)$address, $post->ID );
            }

			// handle attribute
			/*
            $locator_attributes = get_field( 'related-location-attributes', $post->ID, false );
			if ( ! empty( $locator_attributes ) ) {
				foreach ( $locator_attributes as $attribute ) {
					$attribute_post = get_post( $attribute );
					update_field( 'location-attribute-'.$attribute_post->post_name, $attribute_post->post_name, $post->ID );
				}
            }
			*/
		}

		// set cached data and last updated timestamp options
		//update_option( 'bt_locator_attributes', $attributes );
		update_option( 'bt_locator_cache_last_updated', $now );
	//} else {
		//error_log(__FUNCTION__.' -- Got data from cache.');
    }

	return [
        //'attributes' => $attributes,
    ];
}


/**
 * AJAX action: render location locations results (map and list)
 */
add_action( 'wp_ajax_bt_locator', 'bt_locator_locations' );
add_action( 'wp_ajax_nopriv_bt_locator', 'bt_locator_locations' );

/**
 * Query for and build markup (including map point javascript) of locations. This is used by ajax calls but can also be
 * used on a static page where there isn't zip/searching involved so that everything can be cached.
 *
 * @param bool $return If true, return the built markup rather than echoing it, which we'd do if using without ajax
 *
 * @return string|void
 */
function bt_locator_locations( $return = false ) {
	/**
	 * @todo
     * iOS: map recenters on scroll often? (old)
     * Have cached searches like VY site?
	 */

	// get the location post type and make sure we have one
	if ( empty( $location_post_type = bt_locator_post_types() ) ) {
		error_log( __METHOD__ . ': no location post type set in theme setup.' );
		wp_die();
	}

	// get any limit of results we should have
	$count_limit = bt_use_locator( 'count_limit' );
	if ( ! is_numeric( $count_limit ) ) {
		// default, no limit
		$count_limit = 0;
	}

    // verify request
	if ( ! $return ) {
		check_ajax_referer( 'locator_nonce', 'nonce' );
	}

	ini_set('memory_limit', '256M');

	// run data normalization routine without forcing it, which means it will run once per day
	bt_locator_data_normalization( false );

	$request = $_REQUEST; // a copy for debug output
	unset( $request['ajax_url'] );

	// set up common args, no matter the category
	$args = [
		'post_type' => $location_post_type,
		'nopaging' => true,
		'post_status' => 'publish',
		'orderby' => 'title',
		'order' => 'ASC',
		'meta_query' => [
			'relation' => 'AND',
			[
				'key' => 'lat',
				'value' => '',
				'compare' => '!=',
			],
			[
				'key' => 'lng',
				'value' => '',
				'compare' => '!=',
			],
		],
	];

	$location_additional_meta_required = bt_use_locator( 'location_additional_meta_required' );
	if ( is_array( $location_additional_meta_required ) ) {
		//error_log( __METHOD__ . ' -- $location_additional_meta_required = ' . print_r( $location_additional_meta_required, true ) );
		foreach ( $location_additional_meta_required as $meta ) {
			array_push( $args['meta_query'], $meta );
		}
		//error_log( __METHOD__ . ' -- $args[meta_query] = ' . print_r( $args['meta_query'], true ) );
	}

	// get all attributes for use later
	// FILTERS REMOVED $all_attributes = locator_attributes();

	/*/ add keyword search
	$keyword = ( isset( $_REQUEST['keyword'] ) && $_REQUEST['keyword'] ) ? urldecode( $_REQUEST['keyword'] ) : '';
	if ( $keyword ) {
	    $args['s'] = $keyword;
	}*/

	// add zip code and radius filter
	$zip = ( isset( $_REQUEST['zip'] ) && $_REQUEST['zip'] ) ? urldecode( $_REQUEST['zip'] ) : '';
	$zip_lat_lng = $zip ? bt_zip_lat_lng( $zip ) : [];
	$radius = ( isset( $_REQUEST['radius'] ) && $_REQUEST['radius'] ) ? urldecode( $_REQUEST['radius'] ) : '10';

	// validate zip code format
	if ( $zip && ! preg_match( '/^\d{5}$/', $zip ) ) {
		// not a 5-digit string so blank it out
		$zip = '';
	}

	// flag for whether we can do a zip search, and take initial actions based on that
	$doing_zip_filter = $zip && ! empty( $zip_lat_lng ) && $radius;
	if ( $doing_zip_filter ) {
		// add query args which will be used in a custom query
	    $args['lat'] = $zip_lat_lng[0];
	    $args['lng'] = $zip_lat_lng[1];
	    $args['distance'] = $radius;
	    // init params var to use in any results links
		$params = '?';
	/*
	 * THIS COULD BE A THEME-SUPPORTS OPTION (REQUIRING A ZIP TO MAP ANYTHING)
	 *  } else {
		// we don't want to query for anything if we don't have a zip
		echo '<div class="enter-zip"><p>Enter your zip to find Peace Coffee to&nbsp;sip</p></div>';
		wp_die(); */
	}

	// add attributes filter
	/* FILTERS REMOVED
    $filter = [];
    if ( isset( $_REQUEST['filter'] ) && $_REQUEST['filter']) {
        if ( is_numeric( $_REQUEST['filter'] ) ) {
            $filter[] = $_REQUEST['filter'];
        } else if ( is_array( $filter ) ) {
            $filter = $_REQUEST['filter'];
        }
        if ( ! empty( $filter ) ) {
            $filter_clauses = [ 'relation' => 'OR' ];
            foreach ( $filter as $attribute ) {
	            $filter_clauses[] = [
		            'key'       =>  'location-attribute-' . $attribute,
		            'value'     =>  '',
		            'compare'   =>  '!=',
	            ];
	            // add param for link urls within results
	            $params = add_query_arg( $attribute, '1', $params );
            }
            $args['meta_query'][] = $filter_clauses;
        }
    }
	*/

	// add param for zip code in link urls within results
	if ( $doing_zip_filter ) {
		$params = add_query_arg( 'zip', $zip, $params );
	}

    //echo '<pre>request: ' . print_r( $request, 1 ) . '</pre>';
    //echo '<pre>args: ' . print_r( $args, 1 ) . '</pre>';
    //wp_die();

	//error_log('--- ARGS: ' . print_r($args,1));
    $wp_query = $doing_zip_filter ? new WP_Query_BT_Location( $args ) : new \WP_Query( $args );
    $locations = $wp_query->posts;
	$location_count = count( $locations );

	//echo '<pre>locations: ' . print_r( $locations, 1 ) . '</pre>';
    //wp_die();

	// init array of lat/lng pairs so we can track locations at the same point
	$map_locations = [];
	foreach ( $locations as $post ) {
		if ( empty( $post->lat ) || empty( $post->lng ) ) {
			// can't map it
			if ( BT_ENV != 'prod' ) {
				error_log(__FUNCTION__.' -- cannot map post: ' . print_r( $post,1) );
			}
			break;
		}

		// normalize lat/lng
		$post->lat = number_format( $post->lat, 6 );
		$post->lng = number_format( $post->lng, 6 );
		$pair = $post->lat . ',' . $post->lng;
		if ( ! array_key_exists( $pair, $map_locations ) ) {
			$map_locations[ $pair ] = [ $post ];
		} else {
			$map_locations[ $pair ][] = $post;
		}
	}

	// hacky theme filter
	if ( function_exists( '\App\filter_locator_locations' ) ) {
		$map_locations = \App\filter_locator_locations( $map_locations );
	}

	// build up js needed for map data and control, as well as list data if it was requested
	$map_js = 'gmap.point_bounds = new google.maps.LatLngBounds();';
    $list_html = '';
	$report_html = '';
    // loop the locations
    $count = 0;
    foreach ( $map_locations as $pair => $posts ) {
	    $count++;
	    if ( $count_limit && $count > $count_limit ) {
	    	// that's enough
		    break;
	    }
	    //error_log(__FUNCTION__.' -- point ' . $count . ': ' . print_r($point,1));

	    // get the first post at this map point
	    $post = array_shift( $posts );
	    //error_log( __METHOD__ . ': post = ' . print_r( $post, 1 ) );
	    if ( ! is_a( $post, '\WP_Post' ) ) {
	    	continue;
	    }

	    // get address data
	    $address = get_field( 'street_address', $post->ID );
	    if ( empty( $address ) ) {
	    	continue;
	    }
	    //error_log( __METHOD__ . ': post = ' . $post->post_name . ' / address = ' . print_r( $address,1 ) );
	    $street_address_parts = explode( ',', $address['address'] );
	    $street_address = $street_address_parts[0];

	    // format distance
	    if ( $doing_zip_filter ) {
		    $distance = number_format( $post->distance, 1 );
		    if ( $distance == 0 ) {
			    $distance = '0.1';
		    }
		    $distance .= ' miles';
	    } else {
	    	$distance = '';
	    }

	    // build a directions link
	    $directions_link = '<a href="' . bt_directions_url( $address ) . '" target="_blank">Directions</a>';

	    // build icons html, and set flags for what this location has for attributes
	    /* FILTERS REMOVED
	    $icons = '';
	    $beans = false;
	    $brew = false;
	    $location_attributes = get_field( 'related-location-attributes', $post->ID );
	    // Ensure the values for the current location attributes are just the IDs. Imported VenYou data was, but new data is WP_Post objects.
	    array_walk( $location_attributes, function( &$value, $key ) {
	    	if ( is_object( $value ) ) {
	    		$value = $value->ID;
		    }
	    } );
	    foreach ( $all_attributes as $attribute ) {
	    	if ( in_array( $attribute->ID, $location_attributes ) ) {
	    		$icons .= '<abbr class="' . $attribute->post_name . '" title="' . $attribute->post_title . '"><span>&bull;</span></abbr>';
	    		if ( $attribute->post_name == 'fresh-roasted-beans' ) {
				    $beans = true;
			    } else if ( $attribute->post_name == 'fresh-brewed-coffee' ) {
				    $brew = true;
			    }
		    }
	    }
	    */

	    // build info window content - include any remaining posts beyond the first for inclusion
	    $info_window_content = bt_point_info_window( $post, $address, $posts );

	    // determine marker image
	    if ( function_exists( '\App\filter_location_map_pin_path' ) ) {
		    $marker_image = \App\filter_location_map_pin_path( $post, count( $posts ) );
			// get "map-pin-partner-professional-gallery_4bc5e3d9.svg" from "../path/map-pin-partner-professional-gallery_4bc5e3d9.svg"
		    $marker_image_parts = explode( '/', $marker_image );
			// get "map-pin-partner-professional-gallery_4bc5e3d9" from "map-pin-partner-professional-gallery_4bc5e3d9.svg"
		    $marker_image_parts = explode( '.', last( $marker_image_parts ) );
			// get "map-pin-partner-professional-gallery" from "map-pin-partner-professional-gallery_4bc5e3d9"
		    $marker_image_parts = explode( '_', current( $marker_image_parts ) );
			// change "map-pin-partner-professional-gallery" to "map_pin_partner_professional_gallery"
			$marker_type = str_replace( '-', '_', current( $marker_image_parts ) );
		    //error_log( __METHOD__ . " -- {$post->post_type} {$post->post_title} is TYPE $marker_type" );
	    } else {
		    $marker_image = bt_use_locator( 'map_pin_path' );
		    $marker_type = 'pin';
	    }
	    /* FILTERS REMOVED, AND THIS WOULD NEED SOME DATA FROM THEME SUPPORTS
	    if ( $beans && $brew ) {
		    $marker_image = \App\asset_path( 'images/pins/both.png' );
	    } else if ( $brew ) {
		    $marker_image = \App\asset_path( 'images/pins/brew.png' );
	    } else {
		    $marker_image = \App\asset_path( 'images/pins/beans.png' );
	    }
	    */
	    if ( ! empty( $marker_image ) ) {
		    $map_js .= PHP_EOL . 'gmap.marker.url = "' . $marker_image . '";';
	    }

	    // build js for this point
	    $icon = 'gmap.marker';
	    $post_title = addslashes( $post->post_title );
	    $map_js .= <<<JS
var point$count = new google.maps.LatLng($post->lat, $post->lng);
gmap.point_bounds.extend(point$count);
var marker$count = new google.maps.Marker({
    position: point$count,
    map: gmap.map,
    title: "$post_title",
    icon: $icon
});
var pin_$count = function() {
    gmap.info.setContent('<div class="gMapInfo">$info_window_content</div>');
    gmap.info.open(gmap.map, marker$count);
};
google.maps.event.addListener(marker$count, 'click', function() { pin_$count(); });
gmap.markers.push(marker$count);
if ( ! markerGroups.$marker_type ) {
	markerGroups.$marker_type = [];
}
markerGroups.$marker_type.push(marker$count);
JS;

	    // build list content
	    if ( bt_use_locator( 'include_list' ) ) {
	    	$distance_html = $distance ? '<p>' . $distance . '</p>' : '';
		    $list_html .= <<<HTML
<div class="location" data-count="$count">
	<div class="count">$count</div>
	<div class="name">
		<h6>$post->post_title</h6>
		<p>$street_address</p>
	</div>
	<div class="distance">
		$distance_html
		<p>$directions_link</p>
	</div>
</div>
HTML;
		    /* FILTERS REMOVED (.attr was after .name)
		<div class="attr">
			$icons
		</div>
		    */
	    }

    }

    // fit map to point bounds
    $fit_bounds = bt_use_locator( 'map_fit_to_bounds' ) && ! empty( $count )
	    ? 'gmap.map.fitBounds(gmap.point_bounds);'
	    : '';
    $map_js .= <<<JS
var zoomChangeBoundsListener =
    google.maps.event.addListenerOnce(gmap.map, "bounds_changed", function(event) {
        gmap.map.setZoom( Math.min( 15, gmap.map.getZoom() ) );
    });
$fit_bounds
JS;

    // if there weren't any results, we need to center the map
	if ( ! $location_count ) {
		$map_js .= <<<JS
gmap.map.setCenter(new google.maps.LatLng( $zip_lat_lng[0],$zip_lat_lng[1] ));
JS;
	}

  if ( $custom_map_js = bt_use_locator( 'custom_map_js' ) ) {
    $map_js .= <<<JS
$custom_map_js
JS;
  }

    // build report html
	if ( $location_count == 1 ) {
		// singular, use label from theme-supports config if exists
		if ( empty( $location_label = bt_use_locator( 'location_label' ) ) ) {
			$location_label = 'location';
		}
	} else {
		// plural, use label from theme-supports config if exists
		if ( empty( $location_label = bt_use_locator( 'location_label_pl' ) ) ) {
			$location_label = 'locations';
		}
	}
	if ( $location_count ) {
		if ( bt_use_locator( 'include_list' ) ) {
			// some locations are found, report the count
			$report_html = $location_count . ' ' . $location_label;
			if ( $doing_zip_filter ) {
				$report_html .= ' within ' . $radius . ' miles';
			}
			// prepend with a chippy phrase
			/*
			if ( $location_count <= 2 ) {
				$phrases = [
					'Phew! ',
					'Neat! ',
					'Huzzah! ',
				];
				$chippy = $phrases[ array_rand( $phrases ) ];
			} else {
				$chippy = 'Score! ';
			}
			$report_html = $chippy . $report_html;
			*/
			// add note to end of list if there were more results than we show
			if ( $count_limit && $location_count > $count_limit ) {
				$list_html .= '<div class="more-results">' . bt_use_locator( 'count_limit_message' ) . '</div>';
			}
		}
	} else {
		// nothing found in radius, report that
		/*
		$phrases = [
			'Bummer! ',
			'Oh geez! ',
			'Drat! ',
		];
		$chippy = $phrases[ array_rand( $phrases ) ];
		*/
		$report_html .= /* $chippy .*/ 'Nothing in ' . $radius . ' miles. ';
		// determine new radius to try and add link or note
		switch ( $radius ) {
			case '10':
				$new_radius = '25';
				break;
			case '25':
				$new_radius = '50';
				break;
			case '50':
				$new_radius = '100';
				break;
			case '100':
				$new_radius = '';
				break;
			default:
				$new_radius = (string)(10 + intval( $radius ) );
		}
		if ( $new_radius ) {
			$params = add_query_arg( 'radius', $new_radius, $params );
			$report_html .= '<br><br><a id="change-radius" data-new-radius="' . $new_radius . '" href="' . $params . '" class="button btn btn-primary">Try ' . $new_radius . ' Miles?</a>';
		} else {
			$report_html = 'Sorry, there aren\'t any locations within 100 miles.';
		}
	}

	$marker_group_js = <<<JS
var markerGroups = {};
var markerGroupsOff = {};
var toggleMarkerGroup = function(type) {
  if ( Object.keys(markerGroupsOff).length === 0 ) {
    // turn off all except this group
    for ( const group in markerGroups ) {
      var clicked_group = document.getElementById(group);
      if ( group !== type ) {
        toggleMarkerGroupPins(group);
        markerGroupsOff[group] = true;
        clicked_group.classList.add('off');
      } else {
        clicked_group.classList.remove('off');
      }
    }
  } else {
    // toggle current group
    var clicked_group = document.getElementById(type);
    toggleMarkerGroupPins(type);
    if ( typeof markerGroupsOff[type] !== 'undefined' ) {
        delete markerGroupsOff[type];
        clicked_group.classList.remove('off');
    } else {
        markerGroupsOff[type] = true;
        clicked_group.classList.add('off');
    }
  }
};
var toggleMarkerGroupPins = function(type) {
  for (var i = 0; i < markerGroups[type].length; i++) {
    var marker = markerGroups[type][i];
    if (!marker.getVisible()) {
      marker.setVisible(true);
    } else {
      marker.setVisible(false);
    }
  }
};
JS;


    // echo compiled map js, report html, and list html
	$final_html =
		'<script type="text/javascript">'
			. $marker_group_js
			. ( $return
				? 'document.addEventListener("DOMContentLoaded", function() { init_locator(); ' . $map_js . ' } );'
				: $map_js )
			. '</script>'
		. '<div class="report' . ( $location_count ? ' has-results' : '' ) . '">' . $report_html . '</div>'
        . '<div>' . $list_html . '</div>';

	if ( $return ) {
		return $final_html;
	}

	echo $final_html;
	wp_die();
};


/**
 * Generate content markup for a map point's info window.
 *
 * @param \WP_Post $post
 * @param array $address ACF field data
 * @param array $additional_posts When there are multiple posts at one point, these are the ones beyond the first processed.
 *
 * @return string HTML for content, with slashes added.
 */
function bt_point_info_window( $post, $address, $additional_posts = [] ) {
	$location_post_type_name = bt_location_post_type_controller_name( $post );

	// get config items
	$info_win_meta_method = bt_use_locator( 'info_win_meta_method' );
	$info_win_title_class = bt_use_locator( 'info_win_title_class' );
	$info_win_link_title = bt_use_locator( 'info_win_link_title' );
	$info_win_strip_from_address = bt_use_locator( 'info_win_strip_from_address' );
	$info_win_display_phone = bt_use_locator( 'info_win_display_phone' );
	$info_win_display_address = bt_use_locator( 'info_win_display_address' );
	$info_win_display_directions = bt_use_locator( 'info_win_display_directions' );

	// link title for some post types, if array
	if ( is_array( $info_win_link_title ) ) {
		$info_win_link_title = in_array( $post->post_type, $info_win_link_title );
	}

	$html = '<p class="' . implode( ' ', ['title', $info_win_title_class] ) . '">'
	            . ( $info_win_link_title ? '<a href="' . get_the_permalink( $post ) . '">' : '' )
	            . $post->post_title
		        . ( $info_win_link_title ? '</a>' : '' )
	        . '</p>';

	// include meta
	if ( ! empty( $info_win_meta_method ) ) {
		$post_meta = call_user_func_array(
			[
				'\App\Controllers\Partials\Content' . $location_post_type_name,
				$info_win_meta_method,
			],
			[
				$post->ID,
			]
		);
		if ( ! empty( $post_meta ) ) {
			$html .= '<p class="meta">' . $post_meta . '</p>';
		}
	}

	// add any phone numbers
	if ( $info_win_display_phone ) {
		$phone_numbers = get_field( 'phone-numbers', $post->ID );
		if ( ! empty( $phone_numbers ) ) {
			foreach ( $phone_numbers as $phone ) {
				$digits = $phone['area_code'] . '-' . $phone['exchange'] . '-' . $phone['suffix'];
				$html .= '<p class="phone"><a href="tel:' . $digits . '">' . $digits . '</a></p>';
			}
		}
	}

	// add any additional posts at this map point
	if ( is_array( $additional_posts ) && count( $additional_posts ) > 0 ) {
		foreach ( $additional_posts as $additional_post ) {
			$html .= '<p class="' . implode( ' ', ['title', $info_win_title_class] ) . '">'
			        . ( $info_win_link_title ? '<a href="' . get_the_permalink( $additional_post ) . '">' : '' )
			        . $additional_post->post_title
			        . ( $info_win_link_title ? '</a>' : '' )
			        . '</p>';

			// add meta
			if ( ! empty( $info_win_meta_method ) ) {
				$post_meta = call_user_func_array(
					[
						'\App\Controllers\Partials\Content' . bt_location_post_type_controller_name( $additional_post ),
						$info_win_meta_method,
					],
					[
						$additional_post->ID,
					]
				);
				if ( ! empty( $post_meta ) ) {
					$html .= '<p class="meta">' . $post_meta . '</p>';
				}
			}
		}
	}

	// add address and directions link
	if ( $info_win_display_address || $info_win_display_directions ) {
		$html .= '<p class="address">';

		if ( $info_win_display_address ) {
			$display_address = $address['address'];
			// strip out text if set in config
			if ( ! empty( $info_win_strip_from_address ) ) {
				$display_address = str_replace( $info_win_strip_from_address, '', $display_address );
			}
			$html .= $display_address;
		}

		if ( $info_win_display_directions ) {
			$html .= ( $info_win_display_address ? '<br>' : '' )
			         . '<a href="' . bt_directions_url( $address ) . '" target="_blank">Directions</a>';
		}

		$html .= '</p>';
	}

	return addslashes( $html );
}
